% G_CLASS - Classification of all feature files.
% 
% Iterates through all the features files [G_OUTPUT]/feat/*.mat, produces
% classification files [G_OUTPUT]/class/*.mat, and display the results.
% 
% Copyright (C) 2018 ELTE IK NumAnal, GNU GPL 3

function g_class
    g_mkdir('class');
    
    %% files
    files = {};
    list = dir([getenv('G_OUTPUT'), filesep, 'feat', filesep, '*.mat']);
    for i = 1:length(list)
        [~, file, ~] = fileparts(list(i).name);
        files = [files, file];
    end
    
    fprintf('Classification...\n%s\n', sprintf('  %s\n', files{:}));
    parfor i = 1:length(files)
        g_class_file(files{i});
    end
end

function g_class_file(file)
    tic
    %% filenames
    file_feat = [getenv('G_OUTPUT'), filesep, 'feat', filesep, file, '.mat'];
    file_class = [getenv('G_OUTPUT'), filesep, 'class', filesep, file, '.mat'];
    
    %% data load & normalization
    f = load(file_feat);
    n_leads = length(f.train_features);
    lead_config = 1:n_leads;
    for lead = lead_config
        range_features = max(f.train_features{lead}, [], 1) - min(f.train_features{lead}, [], 1);
        range_features(range_features == 0) = 1;

        f.train_features{lead} = f.train_features{lead} ./ (ones(size(f.train_features{lead}, 1), 1) * range_features);
        f.test_features{lead} = f.test_features{lead} ./ (ones(size(f.test_features{lead}, 1), 1) * range_features);
    end
    
    %% classification
    c = g_class_class(f, true);
    save(file_class, '-struct', 'c');

    t = toc;
    fprintf('  %s\t%f%% %f%% (%f%%)  (time: %f s)\n', file, c.accuracy_fused, c.accuracy_rejection, c.p_rejection, t);
end
