% G_FEAT_RAT_POLES_INIT - Initial guess for pole optimization.
% 
% Copyright (C) 2018 ELTE IK NumAnal, GNU GPL 3

function poles = g_feat_rat_poles_init(optim_samples)
    signal = mean(optim_samples, 1);

    %% noise filter
    s = signal;
    w = gausswin(3);
    w = w / sum(w);
    s = filtfilt(w, 1, s);

    %% separate PT
    k1 = 95;
    while k1 > 80 && s(k1) < s(k1+1)
        k1 = k1-1;
    end
    while k1 > 80 && s(k1) > s(k1+1)
        k1 = k1-1;
    end
    s1 = signal(1:k1);
    k2 = 105;
    while k2 < 120 && s(k2) > s(k2+1)
        k2 = k2+1;
    end
    while k2 < 120 && s(k2) < s(k2+1)
        k2 = k2+1;
    end
    s2 = signal(k2:end);

    %% filter
    w = gausswin(15);
    w = w / sum(w);
    ss1 = filtfilt(w, 1, s1);
    ss2 = filtfilt(w, 1, s2);

    %% peaks
    [p1,l1,w1,pr1] = findpeaks(ss1);
    l1 = l1(pr1 == max(pr1));
    [p2,l2,w2,pr2] = findpeaks(ss2);
    l2 = l2(pr2 == max(pr2));

    %% poles
    l2 = k2 + l2 - 1;
    
    if isempty(l1)
        l1 = 50;
    end
    if isempty(l2)
        l2 = 200;
    end
    
    t1 = 2 * pi * l1(end) / 300 - pi;
    t2 = 2 * pi * l2(1) / 300 - pi;
    t0 = 2 * pi * 100 / 300 - pi;
    poles = [0.7*exp(1i*t0) 0.5*exp(1i*t2) 0.5*exp(1i*t1)];
end
