% G_FEAT_RAT_POLES_OPTIM - Pole optimization for rational transform.
% 
% Copyright (C) 2018 ELTE IK NumAnal, GNU GPL 3

function poles = g_feat_rat_poles_optim(samples, pole_mult, poles_init, opts)
    [m, n] = size(samples);
    a0 = [80 120 10]/300;
    a1 = [120 290 80]/300;
    % QRS
    w1 = zeros(m, n);
    w1(:, 80:120) = 1;
    p1 = pole_enc(poles_init(1), a0(1), a1(1));
    p1 = g_simplex(@(x) func(samples, x, pole_mult(1), a0(1), a1(1), w1), p1, opts);
    p1 = pole_dec(p1, a0(1), a1(1));
    [~, ~, appr] = g_feat_rat_mts(samples, p1, pole_mult(1));
    samples = samples - appr;
    % T
    w2 = zeros(m, n);
    w2(:, 120:290) = 1;
    p2 = pole_enc(poles_init(2), a0(2), a1(2));
    p2 = g_simplex(@(x) func(samples, x, pole_mult(2), a0(2), a1(2), w2), p2, opts);
    p2 = pole_dec(p2, a0(2), a1(2));
    % T
    w3 = zeros(m, n);
    w3(:, 10:80) = 1;
    p3 = pole_enc(poles_init(3), a0(3), a1(3));
    p3 = g_simplex(@(x) func(samples, x, pole_mult(3), a0(3), a1(3), w3), p3, opts);
    p3 = pole_dec(p3, a0(3), a1(3));
    % 
    poles = [p1 p2 p3];
end

function err = func(samples, poles, pole_mult, a0, a1, w)
    poles = pole_dec(poles, a0, a1);
    [~, ~, appr] = g_feat_rat_mts(samples, poles, pole_mult);
    e = (samples - appr).^2;
    e = e.*w;
    err = sum(sqrt(sum(e, 2)));
end

function poles = pole_enc(poles, a0, a1)
    r = abs(poles);
    a = angle(poles);
    a = (a + pi) / (2*pi);
    a = (a - a0) ./ (a1 - a0);
    a = 2*pi * a - pi;
    poles = r .* exp(1i*a);
end

function poles = pole_dec(poles, a0, a1)
    r = abs(poles);
    a = angle(poles);
    a = (a + pi) / (2*pi);
    a = (a1 - a0) .* a + a0;
    a = 2*pi * a - pi;
    poles = r .* exp(1i*a);
end
